import { requireNativeComponent, ViewStyle, NativeModules } from 'react-native';
const { CollectorManager } = NativeModules;

export type VGSEnvironment = 'sandbox' | 'live';

type VGSCollectInputProps = {
  config: {
    collectorName: string;
    fieldName: string;
    fieldType: 'cardHolderName' | 'expDate' | 'cvc' | 'cardNumber' | 'text';
  };
  isSecureTextEntry?: boolean;
  placeholder?: string;
  textColor?: string;
  fontFamily?: string;
  fontSize?: number;
  style: ViewStyle;
};

export const VGSCollectInput = requireNativeComponent<VGSCollectInputProps>(
  'VgsCollectReactNativeView'
);

export type SubmitFn = (
  path: string,
  method: 'GET' | 'POST',
  headers: Record<string, string>
) => Promise<{ code: number }>;

export function createNamedCollector(
  name: string,
  vaultId: string,
  environment: VGSEnvironment
): Promise<{
  submit: SubmitFn;
}> {
  return CollectorManager.createNamedCollector(name, vaultId, environment).then(
    () => {
      const submit: SubmitFn = async (path, method = 'POST', headers = {}) => {
        return await CollectorManager.submit(name, path, method, headers);
      };

      return {
        submit,
      };
    }
  );
}
